package internal

import (
	"context"
	"fmt"

	"git.dragse.it/anthrove/otter-space-sdk/pkg/models"
	"github.com/neo4j/neo4j-go-driver/v5/neo4j"
	log "github.com/sirupsen/logrus"
)

func CreateSourceNode(ctx context.Context, driver neo4j.DriverWithContext, anthroveSource *models.AnthroveSource) error {
	query := `
		MERGE (sourceNode:Source {domain: $source_url})
			ON CREATE SET sourceNode.domain = $source_url, sourceNode.display_name = $source_display_name, sourceNode.icon = $source_icon
		`
	params := map[string]any{
		"source_url":          anthroveSource.Domain,
		"source_display_name": anthroveSource.DisplayName,
		"source_icon":         anthroveSource.Icon,
	}

	_, err := neo4j.ExecuteQuery(ctx, driver, query, params, neo4j.EagerResultTransformer)
	if err != nil {
		return fmt.Errorf("graph: %w", err)
	}

	log.WithFields(log.Fields{
		"node_source_url":         anthroveSource.Domain,
		"node_source_displayName": anthroveSource.DisplayName,
		"node_source_icon":        anthroveSource.Icon,
	}).Trace("graph: created source node")

	return nil
}
func GetAllSourceNodes(ctx context.Context, driver neo4j.DriverWithContext) ([]models.AnthroveSource, error) {
	var sources []models.AnthroveSource

	query := `
		MATCH (s:Source)
		RETURN s as source
		`
	params := map[string]any{}

	result, err := neo4j.ExecuteQuery(ctx, driver, query, params, neo4j.EagerResultTransformer)
	if err != nil {
		return nil, err
	}

	if len(result.Records) == 0 {
		return nil, nil
	}

	for i := range result.Records {
		record := result.Records[i]

		source, _, err := neo4j.GetRecordValue[neo4j.Node](record, "source")
		if err != nil {
			return nil, err
		}

		sources = append(sources, models.AnthroveSource{
			DisplayName: source.Props["display_name"].(string),
			Domain:      source.Props["domain"].(string),
			Icon:        source.Props["icon"].(string),
		})

	}

	log.WithFields(log.Fields{
		"tag_amount": len(sources),
	}).Trace("graph: created tag node")

	return sources, nil
}
